/** @file scim_x11_frontend_setup.cpp
 * implementation of Setup Module of X11FrontEnd.
 */

/*
 * Smart Common Input Method
 * 
 * Copyright (c) 2002-2005 James Su <suzhe@tsinghua.org.cn>
 *
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA  02111-1307  USA
 *
 * $Id: scim_x11_frontend_setup.cpp,v 1.15 2005/01/10 08:30:48 suzhe Exp $
 *
 */

#define Uses_SCIM_CONFIG_PATH
#define Uses_SCIM_CONFIG_BASE

#include <iostream>

#include <gtk/gtk.h>

#include "scim_private.h"
#include "scim.h"
#include "scimkeyselection.h"

using namespace scim;

#define scim_module_init x11_frontend_setup_LTX_scim_module_init
#define scim_module_exit x11_frontend_setup_LTX_scim_module_exit

#define scim_setup_module_create_ui       x11_frontend_setup_LTX_scim_setup_module_create_ui
#define scim_setup_module_get_category    x11_frontend_setup_LTX_scim_setup_module_get_category
#define scim_setup_module_get_name        x11_frontend_setup_LTX_scim_setup_module_get_name
#define scim_setup_module_get_description x11_frontend_setup_LTX_scim_setup_module_get_description
#define scim_setup_module_load_config     x11_frontend_setup_LTX_scim_setup_module_load_config
#define scim_setup_module_save_config     x11_frontend_setup_LTX_scim_setup_module_save_config
#define scim_setup_module_query_changed   x11_frontend_setup_LTX_scim_setup_module_query_changed

#define SCIM_CONFIG_FRONTEND_X11_ONTHESPOT             "/FrontEnd/X11/OnTheSpot"
#define SCIM_CONFIG_FRONTEND_X11_BROKEN_WCHAR          "/FrontEnd/X11/BrokenWchar"
#define SCIM_CONFIG_FRONTEND_X11_DYNAMIC               "/FrontEnd/X11/Dynamic"
#define SCIM_CONFIG_FRONTEND_X11_SERVER_NAME           "/FrontEnd/X11/ServerName"

static GtkWidget * create_setup_window ();
static void        load_config (const ConfigPointer &config);
static void        save_config (const ConfigPointer &config);
static bool        query_changed ();

// Module Interface.
extern "C" {
    void scim_module_init (void)
    {
    }

    void scim_module_exit (void)
    {
    }

    GtkWidget * scim_setup_module_create_ui (void)
    {
        return create_setup_window ();
    }

    String scim_setup_module_get_category (void)
    {
        return String ("FrontEnd");
    }

    String scim_setup_module_get_name (void)
    {
        return String (_("X Window"));
    }

    String scim_setup_module_get_description (void)
    {
        return String (_("A FrontEnd module for X11R6 Window System, using XIM Protocol."));
    }

    void scim_setup_module_load_config (const ConfigPointer &config)
    {
        load_config (config);
    }

    void scim_setup_module_save_config (const ConfigPointer &config)
    {
        save_config (config);
    }

    bool scim_setup_module_query_changed ()
    {
        return query_changed ();
    }
} // extern "C"

// Internal data declaration.
static bool __config_on_the_spot           = false;
static bool __config_dynamic               = false;

static bool __have_changed                 = false;

static GtkWidget * __widget_on_the_spot           = 0;
static GtkWidget * __widget_dynamic               = 0;

static GtkTooltips * __widget_tooltips            = 0;

// Declaration of internal functions.
static void
on_default_toggle_button_toggled     (GtkToggleButton *togglebutton,
                                      gpointer         user_data);

static void
setup_widget_value ();

// Function implementations.
static GtkWidget *
create_setup_window ()
{
    static GtkWidget *window = 0;

    if (!window) {
        GtkWidget *table;
        GtkWidget *frame;
        GtkWidget *vbox;
        GtkWidget *label;
        int i;

        __widget_tooltips = gtk_tooltips_new ();

        window = gtk_vbox_new (FALSE, 0);
        gtk_widget_show (window);

        frame = gtk_frame_new (_("XIM Settings"));
        gtk_widget_show (frame);
        gtk_container_set_border_width (GTK_CONTAINER (frame), 4);
        gtk_box_pack_start (GTK_BOX (window), frame, TRUE, TRUE, 0);

        vbox = gtk_vbox_new (FALSE, 0);
        gtk_widget_show (vbox);
        gtk_container_add (GTK_CONTAINER (frame), vbox);
        gtk_container_set_border_width (GTK_CONTAINER (vbox), 4);

        __widget_on_the_spot = gtk_check_button_new_with_mnemonic (_("_On The Spot"));
        gtk_widget_show (__widget_on_the_spot);
        gtk_box_pack_start (GTK_BOX (vbox), __widget_on_the_spot, FALSE, FALSE, 4);

        __widget_dynamic = gtk_check_button_new_with_mnemonic (_("_Dynamic Event Flow"));
        gtk_widget_show (__widget_dynamic);
        gtk_box_pack_start (GTK_BOX (vbox), __widget_dynamic, FALSE, FALSE, 4);

        // Connect all signals.
        g_signal_connect ((gpointer) __widget_on_the_spot, "toggled",
                          G_CALLBACK (on_default_toggle_button_toggled),
                          &__config_on_the_spot);

        g_signal_connect ((gpointer) __widget_dynamic, "toggled",
                          G_CALLBACK (on_default_toggle_button_toggled),
                          &__config_dynamic);

        // Set all tooltips.
        gtk_tooltips_set_tip (__widget_tooltips, __widget_on_the_spot,
                              _("If this option is checked, "
                                "the OnTheSpot input style will be used "
                                "when the client supports it."), NULL);

        gtk_tooltips_set_tip (__widget_tooltips, __widget_dynamic,
                              _("If this option is checked, "
                                "then dynamic event flow will be used."
                                "This can speed up the key event response, "
                                "especially in remote X11 environment."
                                "But SCIM may become unstable."), NULL);

        setup_widget_value ();
    }

    return window;
}

static void
setup_widget_value ()
{
    if (__widget_on_the_spot) {
        gtk_toggle_button_set_active (
            GTK_TOGGLE_BUTTON (__widget_on_the_spot),
            __config_on_the_spot);
    }

    if (__widget_dynamic) {
        gtk_toggle_button_set_active (
            GTK_TOGGLE_BUTTON (__widget_dynamic),
            __config_dynamic);
    }
}

static void
load_config (const ConfigPointer &config)
{
    if (!config.null ()) {
        __config_on_the_spot = 
            config->read (String (SCIM_CONFIG_FRONTEND_X11_ONTHESPOT),
                          __config_on_the_spot);

        __config_dynamic = 
            config->read (String (SCIM_CONFIG_FRONTEND_X11_DYNAMIC),
                          __config_dynamic);

        setup_widget_value ();

        __have_changed = false;
    }
}

static void
save_config (const ConfigPointer &config)
{
    if (!config.null ()) {
        config->write (String (SCIM_CONFIG_FRONTEND_X11_ONTHESPOT),
                       __config_on_the_spot);

        config->write (String (SCIM_CONFIG_FRONTEND_X11_DYNAMIC),
                       __config_dynamic);

        __have_changed = false;
    }
}

static bool
query_changed ()
{
    return __have_changed;
}

static void
on_default_toggle_button_toggled (GtkToggleButton *togglebutton,
                                  gpointer         user_data)
{
    bool *toggle = static_cast<bool*> (user_data);

    if (toggle) {
        *toggle = gtk_toggle_button_get_active (togglebutton);
        __have_changed = true;
    }
}

/*
vi:ts=4:nowrap:expandtab
*/
